package org.ow2.util.ee.metadata.ejbjar.impl.configurator;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;

/**
 * EjbJarArchiveMetadataConfigurator.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Gael Lalire
 */
public abstract class EjbJarDeployableMetadataConfigurator<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        implements IArchiveConfigurator {

    /**
     * archive metadata.
     */
    private D ejbJarArchiveMetadata;

    /**
     * annotationParsingDesactived.
     */
    private boolean annotationParsingDesactived;

    /**
     * Constructor.
     * @param ejbJarArchiveMetadata the archive metadata
     */
    public EjbJarDeployableMetadataConfigurator(final D ejbJarArchiveMetadata) {
        this.ejbJarArchiveMetadata = ejbJarArchiveMetadata;
        this.annotationParsingDesactived = false;
    }

    /**
     * @param jClass signature
     * @return class configurator
     */
    public IClassConfigurator createClassConfigurator(final JClass jClass) {
        return createClassMetadataConfigurator(createClassMetadata(jClass, ejbJarArchiveMetadata),
                this.ejbJarArchiveMetadata, this.annotationParsingDesactived);
    }

    /**
     * Create a field metadataconfigurator for this class configurator.
     * @param classMetadata the given classMetadata
     * @param ejbJarArchiveMetadata the given deployable metadata
     * @param annotationParsingDesactived true if annotation needs to be analyzed
     * @return a new instance of a field configurator
     */
    protected abstract IClassConfigurator createClassMetadataConfigurator(final C classMetadata, D ejbJarArchiveMetadata,
            final boolean annotationParsingDesactived);

    /**
     * @return the archive metadata.
     * @param jClass the given jclass object
     * @param ejbJarArchiveMetadata the given archive metadata
     */
    protected abstract C createClassMetadata(final JClass jClass, final D ejbJarArchiveMetadata);

    /**
     * @return the archive metadata
     */
    public D getEjbJarArchiveMetadata() {
        return this.ejbJarArchiveMetadata;
    }

    /**
     * @return true if annotation parsing is desactived
     */
    public boolean isAnnotationParsingDesactived() {
        return this.annotationParsingDesactived;
    }

    /**
     * Set annotationParsingDesactived.
     * @param annotationParsingDesactived true if you want to desactive annotation parsing
     */
    public void setAnnotationParsingDesactived(final boolean annotationParsingDesactived) {
        this.annotationParsingDesactived = annotationParsingDesactived;
    }

}
